# Generated by Django 3.2.5 on 2022-11-10 10:50

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RequestLogs',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.CharField(max_length=50)),
                ('method', models.CharField(max_length=10)),
                ('path', models.CharField(max_length=100)),
                ('content_type', models.CharField(max_length=50)),
                ('content_params', models.JSONField(default=dict)),
                ('body', models.JSONField(default=dict)),
                ('headers', models.JSONField(default=dict)),
                ('user', models.CharField(max_length=100)),
                ('error', models.TextField(default=None, null=True)),
                ('log_time', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'verbose_name': 'Request Log',
                'verbose_name_plural': 'Request Logs',
            },
        ),
        migrations.CreateModel(
            name='ResponseLogs',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.CharField(default=None, max_length=50, null=True)),
                ('headers', models.JSONField(default=dict)),
                ('content', models.JSONField(default=dict)),
                ('log_time', models.DateTimeField(default=django.utils.timezone.now)),
                ('error', models.TextField(default=None, null=True)),
            ],
            options={
                'verbose_name': 'Response Log',
                'verbose_name_plural': 'Response Logs',
            },
        ),
    ]
